<?php

function generateArraysInternal(&$results, $playerToSet, $playersSet, $playersLeft) {
	$playersLeftNumber = sizeof($playersLeft);
	if ($playersLeftNumber == 0) {
		$results[] = $playersSet;
	} else {
		foreach ($playersLeft as $index => $board) {
			switch ($playerToSet) {
			case 1:
			case 3:
			case 5:
			case 7:
				$add = $board > $playersSet[$playerToSet-1];
				break;
			case 2:
			case 6:
				$add = $board > $playersSet[$playerToSet-2];
				break;
			case 4:
				$add = $board > $playersSet[$playerToSet-4];
				break;
			case 0:
				$add = true;
			}
			if ($add) {
				$nextPlayersSet = $playersSet;
				$nextPlayersLeft = $playersLeft;
				unset($nextPlayersLeft[$index]);
				$nextPlayersSet[$playerToSet] = $board;
				generateArraysInternal($results, $playerToSet+1, $nextPlayersSet, $nextPlayersLeft);
			}
		}
	}
}

function generateArrays() {
	echo 'Generating arrays...';
	$playersLeft = array();
	for ($i=1; $i <= 8; $i++) {
		$playersLeft[] = $i;
	}
	$playersSet = array();
	$results = array();
	generateArraysInternal($results, 0, $playersSet, $playersLeft);
	echo ' '.sizeof($results).' arrays generated'."\n";
	return $results;
}

function getArraySignature($array) {
	return $array[0].'/'.$array[1].' '.$array[2].'/'.$array[3].' '.$array[4].'/'.$array[5].' '.$array[6].'/'.$array[7];
}

function computeScores(&$arrays, $matches, $match4Penalty, $match2Penalty) {
	echo 'Computing scores...';
	$summary = array();
	$ideal = array(1, 8, 4, 5, 2, 7, 3, 6);
	$penaltyMatches4 = array(
		array(0, 1),
		array(2, 3),
		array(4, 5),
		array(6, 7),
		);
	$penaltyMatches2 = array(
		array(0, 2),
		array(0, 3),
		array(1, 2),
		array(1, 3),
		array(4, 6),
		array(4, 7),
		array(5, 6),
		array(5, 7),
		);
	foreach ($arrays as $index => $array) {
		$debug = false;
//		$debug = getArraySignature($array) == '1/5 4/7 2/8 3/6';
		if ($debug) {
			echo 'inspecting array '.getArraySignature($array).'...'."\n";
		}
		$score = 0;
		for ($i=0; $i<8; $i++) {
			$score += abs($array[$i] - $ideal[$i]); 
		}
		if ($debug) {
			echo 'score wrt the ideal cup: '.$score."\n";
		}
		foreach ($matches as $match) {
			$playerA = $match[0];
			$playerB = $match[1];
			if ($debug) {
				echo 'inspecting match '.$playerA.'/'.$playerB.'...'."\n";
			}
			foreach ($penaltyMatches4 as $penaltyMatch) {
				$penaltyPlayerA = $array[$penaltyMatch[0]];
				$penaltyPlayerB = $array[$penaltyMatch[1]];
				if ($debug) {
					echo 'match4: '.$penaltyMatch[0].'/'.$penaltyMatch[1].' => '.$penaltyPlayerA.'/'.$penaltyPlayerB.'...';
				}
				if (($playerA == $penaltyPlayerA && $playerB == $penaltyPlayerB) || ($playerA == $penaltyPlayerB && $playerB == $penaltyPlayerA)) {
					$score += $match4Penalty;
					if ($debug) {
						echo 'penalty: '.$match4Penalty.'!'."\n";
					}
				} else {
					if ($debug) {
						echo 'ok'."\n";
					}
				}
			}
			foreach ($penaltyMatches2 as $penaltyMatch) {
				$penaltyPlayerA = $array[$penaltyMatch[0]];
				$penaltyPlayerB = $array[$penaltyMatch[1]];
				if ($debug) {
					echo 'match2: '.$penaltyMatch[0].'/'.$penaltyMatch[1].' => '.$penaltyPlayerA.'/'.$penaltyPlayerB.'...';
				}
				if (($playerA == $penaltyPlayerA && $playerB == $penaltyPlayerB) || ($playerA == $penaltyPlayerB && $playerB == $penaltyPlayerA)) {
					$score += $match2Penalty;
					if ($debug) {
						echo 'penalty: '.$match2Penalty.'!'."\n";
					}
				} else {
					if ($debug) {
						echo 'ok'."\n";
					}
				}
			}
		}
		$arrays[$index]['score'] = $score;
		if (!isset($summary[$score])) {
			$summary[$score] = 0;
		}
		$summary[$score]++;
	}
	echo ' done'."\n";
	ksort($summary);
	return $summary;
}

function arrayCmp($a, $b) {
	return $a['score'] - $b['score'];
}

function sortArrays(&$arrays) {
	usort($arrays, 'arrayCmp');
}

function printMatches($matches) {
	echo '------------------------------------------'."\n";
	echo 'Matches already played:'."\n";
	if (empty($matches)) {
		echo 'none';
	} else {
		foreach ($matches as $match) {
			echo $match[0].'/'.$match[1].' ';
		}
	}
	echo "\n";
}

function printSummary($summary) {
	echo '------------------------------------------'."\n";
	echo 'Summary:'."\n";
	foreach ($summary as $score => $solutions) {
		echo $score.'pts: '.$solutions.' solution'.($solutions > 1 ? 's' : '')."\n";
	}
}

function printArrays(&$arrays, $max, $names) {
	echo '------------------------------------------'."\n";
	echo $max.' best scores:'."\n";
	$i = 0;
	foreach ($arrays as $index => $array) {
		if ($i == $max) {
			break;
		}
		echo $array['score'].'pts: '.getArraySignature($array)."\n";
		$i++;
	}
	echo '------------------------------------------'."\n";
	$solutionsNumber = 0;
	$bestScore = $arrays[0]['score'];
	foreach ($arrays as $index => $array) {
		if ($array['score'] != $bestScore) {
			break;
		}
		$solutionsNumber ++;
	}
	if ($solutionsNumber > 1) {
		echo $solutionsNumber.' solution'.($solutionsNumber > 1 ? 's' : '').':'."\n";
	} else {
		echo '1 single solution.'."\n";
	}
	$i = 1;
	foreach ($arrays as $index => $array) {
		if ($array['score'] != $bestScore) {
			return;
		}
		if ($solutionsNumber > 1) {
			echo 'Solution #'.$i.':'."\n";
		}
		for ($j=0; $j<4; $j++) {
			echo '  '.$names[$array[2 * $j] - 1].' - '.$names[$array[2 * $j + 1] - 1]."\n";
		}
		$i++;
	}
}

function main() {
	$arrays = generateArrays();
	$names = array(
		'LEJARRE Ludovic',
		'LE GOFF Damien',
		'OTMANI Kamel',
		'ENTEM Alan',
		'LOUIS Corentin',
		'AUBRY Yannis',
		'MINVIELLE Vincent',
		'GEFFROY Leo',
		'MAULANA Kahfi',
		'LE MAREC Simon',
		'JOUNEAU Quentin',
		'CHARPENTIER Maxime',
		);
	$matches = array(
		array(1, 2),
		array(1, 6),
		array(1, 8),
		array(2, 7),
		array(3, 5),
		array(4, 5),
		);
	$summary = computeScores($arrays, $matches, 20, 10);
	sortArrays($arrays);
	printMatches($matches);
	printSummary($summary);
	printArrays($arrays, 10, $names);
}

main();

?>